# -*- coding: utf-8 -*-
"""
Procedures submodule
"""
from .core import *
from .scale import *
from .write import *
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def resting_state_brief_assessment(data=None, language="fr", test=""):
    """
    Standardized Resting State Assessment Procedure.

    Parameters
    ----------
    data = dict
        A dict where to store the data. If None, a new one will be created.

    Returns
    ----------
    None

    Example
    ----------
    None

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame 1.9.2
    """
    # Defining the template of the scale
    def fancy_scale(style, y, title):
        result = scale(style=style, y=y, title=title, title_size=0.6, title_space=1, show_result=True, show_result_space=1, show_result_size=0.35, show_result_shape_size=0.55, show_result_decimals=0)
        return(result)

    # Checking if data is not empty
    if data is None:
        data = {}

    if language == "fr":
        # New blank page
        newpage()
        write("Estimez le pourcentage de temps que vous avez passé dans les 6 états suivants :", y=9.2, size=0.75)

        data[test + "Drowsiness"] = fancy_scale(style="purple", y=7, title="Je me sentais somnolant.")
        data[test + "Absorption_External"] = fancy_scale(style="blue", y=4, title="Mes pensées étaient dirigées sur ce qu'il se passait autour de moi (bruits, voix...).")
        data[test + "Absorption_Internal"] = fancy_scale(style="green", y=1, title="Mes pensées étaient dirigées sur des évenements que j'imaginais ou dont je me rappelais.")
        data[test + "Absorption_Body"] = fancy_scale(style="yellow", y=-2, title="Mes pensées étaient dirigées sur ce qu'il se passait dans mon corps (sensations physiques, respiration, coeur...).")
        data[test + "Mind_Wandering"] = fancy_scale(style="orange", y=-5, title="Mes pensées s'enchainaient librement, sans contrôle.")
        data[test + "Mind_Focus"] = fancy_scale(style="red", y=-8, title="J'étais concentré sur une idée, une sensation ou une perception en particulier.")

    return(data)


# -*- coding: utf-8 -*-
from .path import *
from .core import *
from .image import *
from .write import *
from .miscellaneous import *


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def scale_styles():
    """
    Returns available scale styles.


    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> print(n.scale_styles())
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)
    """
    styles = os.listdir(Path.binary())
    styles = [x for x in styles if "cursor_" in x ]
    styles = [x for x in styles if not "_validated" in x ]
    styles = [x.replace('cursor_', '') for x in styles]
    styles = [x.replace('.png', '') for x in styles]
    return(styles)


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def scale(style='red', x=0, y=-3.3, anchors=None, anchors_space=2, anchors_size=0.7, edges=[0, 100], validation=True, analog=True, step=1, labels="numeric", labels_size=0.8, labels_rotation=0, labels_space=-1, labels_x=0, line_thickness=4, line_length=8, line_color="black", background="white", title=None, title_style="body", title_size=1, title_space=1.75, point_center=False, point_edges=True, reverse=False, force_separation=False, separation_labels=None, separation_labels_size=1, separation_labels_rotate=0, separation_labels_space=-1, show_result=False, show_result_shape="circle", show_result_shape_fill_color="white", show_result_shape_line_color="black", show_result_shape_size=0.8, show_result_space=1.25, show_result_size=0.5, show_result_color="black", show_result_decimals=1, cursor_size=1):
    """
    Draw a scale.

    Parameters
    ----------
    style : str
        style, check `neuropsydia.scale_styles()` function to see what's available.
    x : float
        Horizontal position of the center (from -10 (left) to 10 (right)).
    y : float
        Vertical position of the center (from -10 (bottom) to 10 (top)).
    anchors : list of two str
        a list of two propositions to be displayed on the sides of the scale (e.g., [not at all, very much]).
    anchors_space : float
        spacing betweeen the edge and the anchors.
    anchors_size : float
        size of the anchors' font.
    edges : list
        the underlying numerical edges of the scale.
    validation : bool
        confirm the response with a second left click or withdraw with a right click.
    analog : bool
        continuous (discrete) scale.
    step : int
        if analog is True, what are the step to go between the edges (determine the number of points on the scale).
    labels : str or list
        "num", "numeric" or "numbers" or list of actual text labels (e.g., ["not at all", "a bit", "very much"]).
    labels_size : float
        Size of labels.
    labels_rotation : float
        Labels rotation angle.
    labels_space : float
        Space between scale and labels.
    labels_x : float
        Horizontal dodging position.
    line_thickness : float
        Scale line thickness.
    line_length : float
        Scale line length.
    line_color : str
        Scale line color.
    background : str
        Scale background color.
    title : str
        Scale title/question to ask.
    title_style : str
        'body', 'psychometry', 'psychometry_bold', 'light', 'bold'. You can also insert the name of a system font, or the path to a specific font.
    title_size : float
        Title size.
    title_space : float
        Space between scale and title.
    point_center : bool
        Place a point at the center.
    point_edges : bool
        Place points at the edges.
    reverse : bool
        The result is scored as inverse.
    force_separation : int
        Creates visual separations with points.
    separation_labels : list
        Place labels corresponding to the separations.
    separation_labels_size : float
        Separation labels size.
    separation_labels_rotate : float
        Separation labels rotation angle.
    separation_labels_space : float
        Space between scale and separation labels.
    show_result : bool
        Add a marker to show the value on scale.
    show_result_shape : str
        Shape of this marker. "circle" or "rectangle".
    show_result_shape_fill_color : str
        Fill color of the marker.
    show_result_shape_line_color : str
        Line color of the marker.
    show_result_shape_size : float
        Marker's shape size.
    show_result_space : float
        Space between scale and marker.
    show_result_size : float
        Results text size.
    show_result_color : str
        Results text color.
    show_result_decimals : int
        How many decimals to show.
    cursor_size : float
        Size of the circle cursor.

    Returns
    ----------
    response : float
        Response value.

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> response = n.scale()
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)

    *Dependencies*

    - pygame
    """
    pygame.mouse.set_visible(True)
    # Debugging
    styles_list = scale_styles()
    if style not in styles_list:
        print("NEUROPSYDIA ERROR: scale(): wrong style (check available styles with the scale_styles() function")
        style = "red"

    # change the parameters according  to the style
    cursor_name = 'cursor_' + style + ".png"
    if style == "absorption":
        anchors_space = 3
        anchors = ["Très distant(e)", "Très absorbé(e)"]


    # Coordinates calculating
    line_length_raw = Coordinates.to_pygame(distance_x = line_length)
    scale_y = y  # because y will be further used for the mouse position
    scale_y_raw = Coordinates.to_pygame(y = y) #expressed in pygame's coordinates

    scale_x = x  # center x of the scale
    edge_left = x - line_length /2
    edge_right = x + line_length /2
    edge_left_raw = Coordinates.to_pygame(x = edge_left)
    edge_right_raw = Coordinates.to_pygame(x = edge_right)
    cursor_x = scale_x
    cursor_y = scale_y

    if analog == False:
        #initialize the position and label list
        point_list_range = range(len(range(edges[0],edges[1],step))+1) #is the number of elements in order
        points_position = []
        points_position_raw = [] #positions for points in order to ease the fixation of the cursor
        label_list = []
        #calculate them
        for i in point_list_range:
            points_position.append(edge_left + i*line_length/len(range(edges[0],edges[1],step)))
            points_position_raw.append(edge_left_raw + i*line_length_raw/len(range(edges[0],edges[1],step)))
            if labels == "numeric" or labels == "num" or labels == "numbers":
                label_list.append(str(i))
            elif isinstance(labels, list):
                if len(labels) == len(point_list_range):
                    label_list.append(labels[i])
                else:
                    print("NEUROPSYDIA ERROR: scale(): wrong labels length")
            else:
                print("NEUROPSYDIA ERROR: scale(): labels argument requires a list")

    if separation_labels != None:
        if isinstance(separation_labels, list):
            if force_separation is False:
                force_separation = len(separation_labels)

    def draw_all():
        #Draw the mask
        rectangle(x=scale_x, y=scale_y, width=line_length + anchors_space + 8, height=2, line_color=background, thickness=0, fill_color=background, opacity=225)
        if analog is False:
            rectangle(x=scale_x, y=scale_y + labels_space, width=line_length + anchors_space + 1,height=3, line_color=background, thickness=0,fill_color=background,opacity=225)
        if title != None:
            rectangle(x=scale_x, y=scale_y + title_space, width=line_length*2,height=1,line_color=background,thickness=0,fill_color=background,opacity=225)
        if show_result is True:
            rectangle(x=scale_x, y=y-show_result_space,width=line_length + anchors_space + 8,height=show_result_shape_size+0.7, line_color=background, thickness=0, fill_color=background, opacity=225)
#        Draw the line
        pygame.draw.line(screen, color(line_color), [edge_left_raw,scale_y_raw], [edge_right_raw,scale_y_raw], line_thickness)
        if style == "absorption":
            image('absorption_man.png',x=scale_x, y=scale_y,size=1.3, path=Path.binary())
            image('absorption_desk.png',x=edge_right,y=scale_y,size=1.3, path=Path.binary())
        #draw the anchors
        if anchors != None:
            write(anchors[0], size = anchors_size, x = edge_left - anchors_space,y= scale_y)
            write(anchors[1], size = anchors_size, x = edge_right + anchors_space,y= scale_y)
        #display the points and the labels
        if analog == False:
            for i in point_list_range:
                image('scale_point.png', x=points_position[i], y=scale_y, size=0.25, path=Path.binary())
                write(label_list[i], x=points_position[i]+labels_x, y=scale_y + labels_space, size=labels_size,rotate=labels_rotation)
        else:
            if point_center == True:
                image('scale_point.png', x=scale_x, y=scale_y, size=0.25, path=Path.binary())
            if point_edges == True:
                image('scale_point.png', x=edge_left, y=scale_y, size=0.25, path=Path.binary())
                image('scale_point.png', x=edge_right, y=scale_y, size=0.25, path=Path.binary())
        if separation_labels != None:
            if isinstance(separation_labels, list):
                for i in range(len(separation_labels)):
                    write(separation_labels[i], x=edge_left + line_length/force_separation*(i+1) - line_length/force_separation/2, y=scale_y+separation_labels_space, size=separation_labels_size, rotate=separation_labels_rotate)
            else:
                print("NEUROPSYDIA ERROR: scale(): separation_labels requires a list")
        if force_separation is False:
            if isinstance(force_separation, int):
                for i in range(force_separation-1):
                    image('scale_point.png', x=edge_left + line_length/force_separation*(i+1), y=scale_y, size=0.25)
            else:
                print("NEUROPSYDIA ERROR: scale(): force_separation requires a integer")
        if title != None:
            write(title, title_style, size=title_size, x=scale_x, y=scale_y + title_space)
        image(cursor_name, x=cursor_x, y=scale_y, size=cursor_size, path=Path.binary())
        if show_result is True:
            if show_result_shape == "circle":
                circle(x=cursor_x, y=y-show_result_space, size=show_result_shape_size, fill_color=show_result_shape_fill_color, line_color=show_result_shape_line_color)
            if show_result_shape == "rectangle":
                rectangle(x=cursor_x, y=y-show_result_space, width=show_result_shape_size, height=show_result_shape_size, fill_color=show_result_shape_fill_color, line_color=show_result_shape_line_color)
            if show_result_decimals == 0:
                write("%.0f" %(edges[0] + (cursor_x - edge_left)*(edges[1]-edges[0])/line_length), x=cursor_x, y=y-show_result_space, size=show_result_size, color=show_result_color)
            if show_result_decimals == 1:
                write("%.1f" %(edges[0] + (cursor_x - edge_left)*(edges[1]-edges[0])/line_length), x=cursor_x, y=y-show_result_space, size=show_result_size, color=show_result_color)
            if show_result_decimals == 2:
                write("%.2f" %(edges[0] + (cursor_x - edge_left)*(edges[1]-edges[0])/line_length), x=cursor_x, y=y-show_result_space, size=show_result_size, color=show_result_color)
        refresh()
    pygame.event.set_allowed(pygame.KEYDOWN)


    draw_all()
    loop=True
    while loop is True:
        for event in pygame.event.get(): #for each event...
            cursor_x_raw, cursor_y_raw = pygame.mouse.get_pos()  #.get mouse position
            cursor_x, cursor_y = Coordinates.from_pygame(x=cursor_x_raw, y=cursor_y_raw)  # convert the raw position


            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return(np.nan)
                if event.key in keys["normal"]:
                    if keys["normal"][event.key] == "LEFT" or keys["normal"][event.key] == "RIGHT":
                        pygame.event.set_blocked(pygame.KEYDOWN)
                        return(keys["normal"][event.key])
            if cursor_y < scale_y+2 and cursor_y > scale_y-2:
                if cursor_x < edge_right + 2 and  cursor_x > edge_left - 2:
                    #fix the x
                    if cursor_x < edge_left:
                        cursor_x = edge_left
                    if cursor_x > edge_right:
                        cursor_x = edge_right
                    if analog == False:
                        cursor_x = min(points_position, key=lambda a:abs(a-cursor_x))

                    draw_all()

                    if pygame.mouse.get_pressed()==(1,0,0):
                        #convert the  real ratio into the reponse
                        response = edges[0] + (cursor_x - edge_left)*(edges[1]-edges[0])/line_length

                        #what happened when validation is on
                        if validation == True:
                            cursor_name = 'cursor_' + style + '_validated.png'
                            draw_all()

                            mouse_pressed = True
                            while mouse_pressed == True:
                                for event in pygame.event.get():
                                    if event.type == pygame.KEYDOWN:
                                        if event.key == pygame.K_ESCAPE:
                                            return(np.nan)
                                        if event.key in keys["normal"]:
                                            if keys["normal"][event.key] == "LEFT" or keys["normal"][event.key] == "RIGHT":
                                                pygame.event.set_blocked(pygame.KEYDOWN)
                                                return(keys["normal"][event.key])
                                    if pygame.mouse.get_pressed()==(1,0,0):
                                        time.wait(100)
                                        mouse_pressed = False
                                        loop = False
                                    if pygame.mouse.get_pressed()==(0,0,1):
                                        cursor_name = 'cursor_' + style + '.png'
                                        mouse_pressed = False
    if reverse == True:
        response = -1*(response - (edges[0]+edges[1]))
    pygame.mouse.set_visible(False)
    pygame.event.set_blocked(pygame.KEYDOWN)
    return(response)


# -*- coding: utf-8 -*-
from .path import *
from .core import *
from .write import *



#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
#==============================================================================
def choice(choices=["Yes","No"], write_choices=True, overwrite_choices_display=None, choices_size=1.0, choices_color="black", y=0, height=-2, boxes_space=0.5, boxes_background='white', boxes_edge_color="black", boxes_edge_size=3, confirm_edge_color="orange", confirm_edge_size=3, help_list=None, help_background="lightgrey", title=None, title_position="top", title_x=-7.5, title_space=0.75, pictures=None, pictures_size=0.5):
    """
    Create clickable choice boxes.

    Parameters
    ----------
    choices : list
        List of choices.
    write_choices : bool
        Write choices inside the boxes.
    overwrite_choices_display : list
        Display different choices (but does not affect what is returned).
    choices_size : float
        Choices text size.
    choices_color : str
        Choices text color.
    y : float
        Vertical position.
    height : float
        Boxes height (Should be negative).
    boxes_space : float
        Spaces between boxes.
    boxes_background : str
        Boxes background color.
    boxes_edge_color : str
        Boxes edges color.
    boxes_edge_size : float
        Boxes edges width.
    confirm_edge_color : str
        When clicked once, to what color should the edges change?
    confirm_edge_size : float
        When clicked once, to what width should the edges change?
    help_list : list
        Display some additional text for each choice.
    help_background : str
        Background color of the help band.
    title : str
        Title text.
    title_position : str
        Title position. "top" or "left".
    title_x : float
        Title horizontal position.
    title_space : float
        Space between choices and title.
    pictures : list
        Picture filenames (and path) to put within each box.
    pictures_size : float
        Size of those pictures.

    Returns
    ----------
    response : str or float
        The selected choice.

    Example
    ----------
    >>> import neuropsydia as n
    >>> n.start()
    >>> answer = n.choice(["No", "Maybe", "Yes"])
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)

    *Dependencies*

    - pygame
    """


    raw_y = y

    distance_y=height
    number = len(choices)

    if number == 2 and title_position == "default":
        title_position = "left"
    if number != 2 and title_position == "default":
        title_position = "top"
    previous_mouse_state = pygame.mouse.set_visible(True)


    list_x = [-10+boxes_space/2]
    for i in range(number-1):
        list_x.append(list_x[i]+  (20-boxes_space*number)/number + boxes_space)
    list_x2 = []
    for i in list_x:
        list_x2.append(Coordinates.to_pygame(x=i))


    coordinates = {
    'x':list_x2,
    'y':number*[Coordinates.to_pygame(y=y)],
    'width_raw':(20-boxes_space*number)/number,
    'height_raw':distance_y,
    'width':Coordinates.to_pygame(distance_x=(20-boxes_space*number)/number),
    'height':Coordinates.to_pygame(distance_y=distance_y)
    }

    if number == 2:
        coordinates = {
    'x':[Coordinates.to_pygame(x=-5.1),Coordinates.to_pygame(x=0.6)],
    'y':number*[Coordinates.to_pygame(y=y)],
    'width_raw':4.4,
    'height_raw':distance_y,
    'width':Coordinates.to_pygame(distance_x=4.4),
    'height':Coordinates.to_pygame(distance_y=distance_y)
    }

    def Display():

        for i in range(number):
            if isinstance(boxes_background, list) == True:
                pygame.draw.rect(screen, color(boxes_background[i]), (coordinates['x'][i],coordinates['y'][i],coordinates['width'],coordinates['height']),0)
            elif isinstance(color(boxes_background), list) == True:
                pygame.draw.rect(screen, color(boxes_background)[i], (coordinates['x'][i],coordinates['y'][i],coordinates['width'],coordinates['height']),0)
            else:
                pygame.draw.rect(screen, color(boxes_background), (coordinates['x'][i],coordinates['y'][i],coordinates['width'],coordinates['height']),0)
            if boxes_edge_size != 0:
                pygame.draw.rect(screen, color(boxes_edge_color), (coordinates['x'][i],coordinates['y'][i],coordinates['width'],coordinates['height']), boxes_edge_size)
            if write_choices is True:
                if overwrite_choices_display is None:
                    write(choices[i],x=Coordinates.from_pygame(coordinates['x'][i])+ coordinates['width_raw']/2,y=Coordinates.from_pygame(y=coordinates['y'][i])+coordinates['height_raw']/2,color=choices_color,size=choices_size)
                if isinstance(overwrite_choices_display, list):
                    write(overwrite_choices_display[i],x=Coordinates.from_pygame(coordinates['x'][i])+ coordinates['width_raw']/2,y=Coordinates.from_pygame(y=coordinates['y'][i])+coordinates['height_raw']/2,color=choices_color,size=choices_size)
            if isinstance(pictures,list):
                image(pictures[i],x=Coordinates.from_pygame(coordinates['x'][i])+ coordinates['width_raw']/2,y=Coordinates.from_pygame(y=coordinates['y'][i])+coordinates['height_raw']/2,size=pictures_size)
        if title != None:
            if title_position == 'left':
                write(title,x=title_x,y=Coordinates.from_pygame(y=coordinates['y'][0])+coordinates['height_raw']/2)
            if title_position == 'top':
                write(title,y=Coordinates.from_pygame(y=coordinates['y'][0])+title_space)
        pygame.display.flip()




    Display()
    loop = True
    while loop == True:
        for event in pygame.event.get():
            if event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE:
                quit()
            x, y = pygame.mouse.get_pos()
            if help_list != None:
                if y > coordinates['y'][0] and y < (coordinates['y'][0]+coordinates['height']):
                        for i in range(number):
                            if x < (coordinates['x'][i]+coordinates['width']) and x > coordinates['x'][i]:
                                pygame.draw.rect(screen, color(help_background), (Coordinates.to_pygame(x=-10),coordinates['y'][i]+coordinates['height']+Coordinates.to_pygame(distance_y=-0.25),screen_width,Coordinates.to_pygame(distance_y=-1.50)),0)
                                write(help_list[i],y=raw_y+coordinates['height_raw']-1)
                                pygame.display.flip()



            if pygame.mouse.get_pressed()==(1,0,0):
                if y > coordinates['y'][0] and y < (coordinates['y'][0]+coordinates['height']):
                    for i in range(number):
                        if x < (coordinates['x'][i]+coordinates['width']) and x > coordinates['x'][i]:
                            response = i
                            pygame.draw.rect(screen, color(confirm_edge_color), (coordinates['x'][response],coordinates['y'][response],coordinates['width'],coordinates['height']), confirm_edge_size)
                            pygame.display.flip()
                            time.wait(100)
                            loop2 = True
                            while loop2 == True:
                                for event in pygame.event.get():
                                    if pygame.mouse.get_pressed()==(1,0,0):
                                        loop2= False
                                        loop = False
                                    if pygame.mouse.get_pressed()==(0,0,1):
                                        loop2=False
                                        Display()
#                                        pygame.draw.rect(screen, color('b'), (coordinates['x'][response],coordinates['y'][response],coordinates['width'],coordinates['height']), boxes_edge_size)
                                        pygame.display.flip()

    pygame.draw.rect(screen, color('green'), (coordinates['x'][response],coordinates['y'][response],coordinates['width'],coordinates['height']), confirm_edge_size)
    pygame.display.flip()
    pygame.mouse.set_visible(previous_mouse_state)
    return(choices[response])








from unittest import TestCase

import neuropsydia as n
n.start(open_window=False)

class TestColor(TestCase):
    def test_is_string(self):
        c = n.color("w")
        self.assertTrue(isinstance(c, tuple))


# -*- coding: utf-8 -*-
import neurokit as nk
import pandas as pd
import datetime
import random

from .path import *
from .core import *
from .write import *
from .image import *
from .start import *
from .scale import *
from .ask import *


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def instructions(text, background='white', color="black", size=1.0,
                 title="INSTRUCTIONS", title_color="black", subtitle=None, subtitle_color="black", end_text="Appuyez sur ENTRER pour commencer.", top_space=5):
    """
    Help incomplete, sorry.

    Parameters
    ----------
    NA

    Returns
    ----------
    NA

    Example
    ----------
    NA

    Authors
    ----------
    Dominique Makowski

    Dependencies
    ----------
    - pygame 1.9.2
    """
    newpage(background, auto_refresh=False)

    if title is not None:
        write(title, style="title", color=title_color)
    if subtitle is not None:
        write(subtitle, style="subtitle", color=subtitle_color)

    top_space = ["\n"]*top_space
    top_space = "".join(str(elem) for elem in top_space)
    write(top_space + text, size=size, color=color, long_text=True)
    write(end_text, style='end', color=color)


# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
# ==============================================================================
def questionnaire(questions_dictionary, questions_list_key_name='Item', background='white', size=1, show_page_number=True,  randomize=False, reverse=False, results_save=False, results_name="questionnaire_results", results_path="", participant_id="", dimensions_mean=False, dimensions_key_name='Dimension', style='red', x=0, y=-3.3, anchors=None, anchors_space=2, anchors_size=0.7, edges=[0,100], validation=True, analog=True, step=1, labels="numeric", labels_size=0.8, labels_rotation=0, labels_space=-1, labels_x=0, line_thickness=4, line_length=8, line_color="black", title=None, title_style="body", title_size=1, title_space=2, point_center=False, point_edges=True, force_separation=False, separation_labels=None, separation_labels_size=1, separation_labels_rotate=0, separation_labels_space=-1, show_result=False, show_result_shape="circle", show_result_shape_fill_color="white", show_result_shape_line_color="red", show_result_shape_size=0.8, show_result_space=1.2, show_result_size=0.5, show_result_color="black", instructions_text=None, instructions_top_space=5, show_result_decimals=1, cursor_size=1):
    """
    A wrapper function for easily creating questionnaires. You can go back and forth in the questions using the LEFT and RIGHT keyboard arrows.

    Parameters
    ----------
    questions_dictionary : dict
        A dict of the following stucture:

        >>> questions_dictionary = {
        >>>     "Item": {
        >>>         1: "Is Neuropsydia great?",
        >>>         2: "Is Neuropsydia not great?",
        >>>         3: "Is Python great?",
        >>>         4: "Is Python not great?"
        >>>         }
        >>> }

    questions_list_key_name : str
        Key name of the sub-dict containing the items.
    background : str
        Background color.
    size : int
        Question's size.
    show_page_number : bool
        Show page number.
    randomize : bool
        Randomize question presentation.
    reverse : bool
        Needs a "Reverse" sub-dict with booleans showing which questions are reversed.
    results_save : bool
        Save the results.
    results_name : str
        Filename.
    results_path : str
        Path where to save.
    participant_id : str
        Append the participant's name in the filename.
    dimensions_mean : bool
        Compute the mean by dimension. Needs a "Dimension" sub-dict.
    dimensions_key_name : str
        Key name of the sub-dict containing dimensions.
    style : str
        style, check `neuropsydia.scale_styles()` function to see what's available.
    x : float
        Horizontal position of the center (from -10 (left) to 10 (right)).
    y : float
        Vertical position of the center (from -10 (bottom) to 10 (top)).
    anchors : list of two str
        a list of two propositions to be displayed on the sides of the scale (e.g., [not at all, very much]).
    anchors_space : float
        spacing betweeen the edge and the anchors.
    anchors_size : float
        size of the anchors' font.
    edges : list
        the underlying numerical edges of the scale.
    validation : bool
        confirm the response with a second left click or withdraw with a right click.
    analog : bool
        continuous (discrete) scale.
    step : int
        if analog is True, what are the step to go between the edges (determine the number of points on the scale).
    labels : str or list
        "num", "numeric" or "numbers" or list of actual text labels (e.g., ["not at all", "a bit", "very much"]).
    labels_size : float
        Size of labels.
    labels_rotation : float
        Labels rotation angle.
    labels_space : float
        Space between scale and labels.
    labels_x : float
        Horizontal dodging position.
    line_thickness : float
        Scale line thickness.
    line_length : float
        Scale line length.
    line_color : str
        Scale line color.
    background : str
        Scale background color.
    title : str
        Scale title/question to ask.
    title_style : str
        'body', 'psychometry', 'psychometry_bold', 'light', 'bold'. You can also insert the name of a system font, or the path to a specific font.
    title_size : float
        Title size.
    title_space : float
        Space between scale and title.
    point_center : bool
        Place a point at the center.
    point_edges : bool
        Place points at the edges.
    reverse : bool
        The result is scored as inverse.
    force_separation : int
        Creates visual separations with points.
    separation_labels : list
        Place labels corresponding to the separations.
    separation_labels_size : float
        Separation labels size.
    separation_labels_rotate : float
        Separation labels rotation angle.
    separation_labels_space : float
        Space between scale and separation labels.
    show_result : bool
        Add a marker to show the value on scale.
    show_result_shape : str
        Shape of this marker. "circle" or "rectangle".
    show_result_shape_fill_color : str
        Fill color of the marker.
    show_result_shape_line_color : str
        Line color of the marker.
    show_result_shape_size : float
        Marker's shape size.
    show_result_space : float
        Space between scale and marker.
    show_result_size : float
        Results text size.
    show_result_color : str
        Results text color.
    show_result_decimals : int
        How many decimals to show.
    cursor_size : float
        Size of the circle cursor.

    Returns
    ----------
    df : pandas.DataFrame
        A pandas' dataframe containing the data.

    Example
    ----------
    >>> import neuropsydia as n
    >>> questions_dictionary = {
    >>>     "Item": {
    >>>         1: "Is Neuropsydia great?",
    >>>         2: "Is Neuropsydia not great?",
    >>>         3: "Is Python great?",
    >>>         4: "Is Python not great?"
    >>>     },
    >>>     "Reverse": {
    >>>         1: False,
    >>>         2: True,
    >>>         3: False,
    >>>         4: True
    >>>     },
    >>>     "Dimension": {
    >>>         1: "Neuropsydia",
    >>>         2: "Neuropsydia",
    >>>         3: "Python",
    >>>         4: "Python"
    >>>     }
    >>> }
    >>> n.start()
    >>> n.questionnaire(questions_dictionary, anchors=["No", "Yes"], results_save=True, dimensions_mean=True)
    >>> n.close()

    Notes
    ----------
    *Authors*

    - Dominique Makowski (https://github.com/DominiqueMakowski)

    *Dependencies*

    - pygame
    - pandas
    """
    time_start = datetime.datetime.now()
    if isinstance(questions_dictionary,dict) == False:
        print("NEUROPSYDIA ERROR: questionnaire(): wrong object given (not a dictionary), check http://www.neuropsydia.com/#!create-questionnaire/ncq23 for help.")
    try:
        questions = questions_dictionary[questions_list_key_name]
    except:
        print("NEUROPSYDIA ERROR: questionnaire(): arg questions_list_key_name does not match.")

    n_questions = len(questions)
    order = list(questions.keys())
    if randomize is True:
        random.shuffle(order)

    questions_dictionary["Answer"] = {}
    questions_dictionary["RT"] = {}
    questions_dictionary["Order"] = {}
    questions_dictionary["Time"] = {}

    if instructions_text != None:
        try:
            instructions(instructions_text, top_space=instructions_top_space)
        except:
            print("NEUROPSYDIA ERROR: questionnaire(): problem in 'instructions_text' arg, make sure it's a normal string.")



    i = 0
    while i < n_questions:
        try:
            newpage(background, auto_refresh=False)
            time.wait(30)
            if show_page_number is True:
                write('Question n°' + str(i+1) + '/' + str(n_questions), style="light", y=9)
            text = questions_dictionary[questions_list_key_name][order[i]]
            write('\n\n\n\n\n\n'+text, long_text=True, size=size)
            try:
                reverse_question = questions_dictionary['Reverse'][order[i]]
            except:
                if isinstance(reverse,list) is True:
                    reverse_question=reverse[i]
                else:
                    reverse_question=reverse

            try:
                if isinstance(questions_dictionary["Anchors"], dict):
                    anchors = questions_dictionary['Anchors'][order[i]]
            except:
                pass

            t0 = builtin_time.clock()
            answer = scale(style=style, anchors=anchors, anchors_space=anchors_space, anchors_size=anchors_size, edges = edges, validation=validation, analog=analog, step=step, labels=labels, labels_size=labels_size, labels_rotation=labels_rotation, labels_space=labels_space,labels_x=labels_x, line_thickness=line_thickness, line_length=line_length, line_color=line_color, background=background, title=title, title_style=title_style, title_size=title_size, title_space=title_space, reverse=reverse_question, point_center=point_center, point_edges=point_edges, force_separation=force_separation, separation_labels=separation_labels, separation_labels_size=separation_labels_size, separation_labels_rotate=separation_labels_rotate, separation_labels_space=separation_labels_space, show_result=show_result, show_result_shape=show_result_shape, show_result_shape_fill_color=show_result_shape_fill_color, show_result_shape_line_color=show_result_shape_line_color, show_result_shape_size=show_result_shape_size, show_result_space=show_result_space, show_result_size=show_result_size, show_result_color=show_result_color, show_result_decimals=show_result_decimals, cursor_size=cursor_size)
            RT = (builtin_time.clock()-t0 ) * 1000
            if answer == 'RIGHT':
                answer = np.nan
            questions_dictionary["Answer"][order[i]] = answer
            questions_dictionary["RT"][order[i]] = RT
            questions_dictionary["Order"][order[i]] = i+1
            questions_dictionary["Time"][order[i]] = datetime.datetime.now()
            if answer == 'LEFT':
                i -= 1
            else:
                i += 1
        except:
           print("NEUROPSYDIA ERROR: questionnaire(): someting went wrong (error number 2)")

    df = pd.DataFrame.from_dict(questions_dictionary)
    df["Time_Start"] = time_start
    df["Time_End"] = datetime.datetime.now()
    df["Total_Duration"] = (datetime.datetime.now()-time_start).total_seconds()

    if dimensions_mean == True:
        for dim in set(df[dimensions_key_name]):
            df[dim] = df[(df[dimensions_key_name]==dim)]["Answer"].mean()

    if results_save == True:
        nk.save_data(df, filename=results_name, path=results_path, participant_id=participant_id, index=True, index_label="Item_Number")

    return(df)
